@@ -3,20 +3,28 @@ import json
 import math
 from abc import ABC
 from concurrent import futures
+import asyncio
+import aiohttp
 from logging import getLogger
-from typing import Dict, List, TypeVar, Type, Union, Optional
+from typing import Dict, List, TypeVar, Type, Union, Optional, Tuple
 
 import tqdm
 import urllib3
+from aiologger.levels import LogLevel
+from coloredlogs import ColoredFormatter
 from pydantic.json import pydantic_encoder
 from requests import Session, Response
 from requests_futures.sessions import FuturesSession
-
+from aiologger import Logger
 from rallycli.errors import RallyError, requests_error_handling, rally_raise_errors
 from rallycli.models import RallyTypeGeneric
 
 ## logger definition
 logger = getLogger(__name__)
+## async logger
+cfrm = ColoredFormatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
+aiologger = Logger.with_default_handlers(name=__name__, level=LogLevel.INFO)
+
 ## Disable certificate warnings for testing pourposes
 urllib3.disable_warnings()
 
@@ -120,6 +128,38 @@ class BaseAPI(ABC):
             root_element_name = list(response.json().keys())[0]
             return response.json().get(root_element_name)
 
+    async def _async_post(self, session: aiohttp.ClientSession, url: str, data: dict) -> dict:
+        async with session.post(url, data=json.dumps(data, default=pydantic_encoder)) as response:
+            jresults = await response.json(content_type=None)
+            # logger.info(f"Post result ok: {response.ok}")
+            response.raise_for_status()
+            if jresults.get("OperationResult") and jresults.get("OperationResult").get("Object"):
+                aiologger.debug(f"Updating {url}: {response.ok}")
+                return jresults["OperationResult"]["Object"]  # udpate/delete
+            elif jresults.get("OperationResult") and jresults.get("OperationResult").get("Results"):
+                return jresults["OperationResult"]["Results"]  # Collections
+            elif jresults.get("CreateResult") and jresults.get("CreateResult").get("Object"):
+                aiologger.debug(f"Creating {url}: {jresults['CreateResult']['Object']['Name']}")
+                return jresults["CreateResult"]["Object"]  # Create
+            else:
+                logger.debug(jresults)
+                root_element_name = list(jresults.keys())[0]
+                return jresults.get(root_element_name)
+
+    @requests_error_handling
+    async def _async_post_elements(self, url_data: List[Tuple[str, dict]]) -> List[dict]:
+        headers = self._rally_session.headers
+        tasks: List[asyncio.Task] = []
+        async with aiohttp.ClientSession(headers=headers) as session:
+            for t in url_data:
+                url: str = t[0]
+                data: dict = t[1]
+                tasks.append(asyncio.create_task(self._async_post(session, url, data)))
+            aiologger.info(f"Wainting (gather) for {len(tasks)}")
+            result_list: List[dict] = await asyncio.gather(*tasks)
+
+            return result_list
+
     @requests_error_handling
     def _get(self, url: str,
              params: dict,
